================================================================================
B A S T A R D                                            disassembly environment



Intro...
Well, this is going to double as a manual and a spec ... bear with.

================================================================================
 QuickStart

The most common use of the bastard is to load an x86 ELF file and perform a
full disassembly on it; this is done by the 'LOAD' command [aliased to 'L'
to save typing]. The file can then be printed to STDOUT using the 'DISASM'
command [or 'D']. The following demonstrates this as performed on the file
/usr/bin/tr:

   [/home/work/bastard]$>./bastard
	;>l /usr/bin/tr
	;>d

Notice how the output fills the screen buffer rather quickly. The command
line has a pipe ['|'] metacharacter that will pass the output from a bastard
command to a shell command line; thus, it is possible to send output to 
various unixland commands:

	[/home/work/bastard]$>./bastard
	;>l /usr/bin/tr
	;>d | more
	;>d | wc -l
	;>d | grep call | head

The file can be saved using the SaveAsASM() command:

	[/home/work/bastard]$>./bastard
	;>l /usr/bin/tr
	;>SaveAsASM("tr.asm");

Disassembled files are saved in .bdb database files when unloaded, when
exitting the disassembler, or by calling SaveDB() or SaveDBAs() :

	[/home/work/bastard]$>./bastard
	;>l /usr/bin/tr
	;>SaveDBAs("tmp.bdb");
	;>q
	[/home/work/bastard]$>./bastard
	;>LoadBDB("tmp.bdb");
	;>d

This last example also introduces the highly influential 'QUIT' command, or
'q' for short. Additional commands can be found by typing 'HELP' or '?'; the
more interesting ones are:

	DEBUG    -- Run the target in gdb
	EXEC     -- The good old shell escape
	HEADER   -- Print the program header
	SECTION  -- Print the disassembly of a specific section
	STRINGS	-- Print all strings found in the target

Note that the DB can be examined from the command line with the DB command 
set:

	DB HELP
	DB COUNT [table]       -- Count number of records in 'table'
	DB DESC  [table]       -- Print the description [ala struct] of 'table'
	DB DUMP  [table]       -- Dump the contents of 'table'
	DB MAX   [table] [key] -- Print the maximum value of column 'key' in 'table'
	DB MIN   [table] [key] -- Print the minimum value of column 'key' in 'table'

In case you haven't guessd it by now, the pipe feature was added in order to
make dealing with the DB a lot easier.


The intent of the bastard is by and large to be controlled by a shell script
or a parent process [read "front end"]. The bastard is fully scriptable via
the command line, with all of its internal API commands as well as an
embedded C interpreter available via STDIN. As such, it can be controlled
simply by redirecting its input from a file:
   
	./bastard < commands.in > asm.out

Some sample shell scripts for controlling the bastard are provided in the 
directory $BASTARD_HOME/utils :

	utils/disasm.color.sh      -- demonstrates the 'colorized' mode  
	utils/disasm.full.sh       -- does a full disassembly
	utils/disasm.libopcodes.sh -- forces the use of libopcodes [bad idea]
	utils/disasm.text.sh       -- disassembles the .text section



================================================================================
 Theory of Operation

The action all begins when LoadTarget() is used to open a file to disassemble. 
In the future, support may be provided for binary input via STDIN; this will
be made available through a command line option, and may provide the ability
to disassemble arbitrary sequences of bytes.

A .bdb directory is created in the current directory; this will contain the
database and temporary files needed during disassembly. When the database is
saved, the contents of .bdb will be tar-gz'ed into a file with a '.bdb'
extension. The target is copied to a binary image inside .bdb [1], a Target
database is created, and a record is added to the ADDRESS object table with
ADDRESS->PA = 0 and ADDRESS->size = sizeof(target). A TARGET struct is created
as well:
	struct TARGET{
		char* name,
		char* path,
		long  size,
		ADDR  entry;
	} Target;

The next step is to apply a file format structure to the file -- since an
executable is in essence a predictable structure composed of blocks of code and
data, different file formats will be considered different structures rather
than different disassembly operations. Hopefully this will the support for
future file formats more straightforward.

The structure definition will be a Seer C script containing the structure
definition, and a brief main routine for creating sections:
 /* File Format Sample ------------------------------------------------ */
	import int  FD;
	import int  read(int fd, void *buf, int count);
	import void malloc(int size);
	import void free(void* ptr);
	import void CreateSection(char* name, long start, long end, int flags);
	export main;
	struct FILEFMT{
		....
	} FileHeader;
	struct FILESEC{
		....
	} FileSection;

	void main() {
		int flags;
		char ptr = malloc( sizeof(FILEFMT) );
		read( FD, ptr, sizeof(FILEFMT));
		for (/* each section */) {
			CreateSection( Section.Name, Section.start, Section.End, flags);
		}
 /* EOF ----------------------------------------------------------------*/

As you can see, the code for parsing the header is actually part of this
script -- this allows arbitrarily complex programs or files to be ported
without recompiling the disassembler. This could be used, in theory, for
writing unpackers.

When the File Format is selected using SetFileType(), the Seer script for the
specified type is executed using a file descriptor to the binary image as a
global variable. A record is added to the SECTION table of the database for
each section created with CreateSection; likewise, an ADDRESS record is added 
and sized for the start of each section. [2]

At this point, the target can be considered 'loaded' and ready for disassembly.
There are many disassembly options available, but in general one will begin
with a disassembly of whichever section contains the program entry point, 
starting of course at that entry point:

		

Notes:
	1. All of the bytes in the file must be retained in order to produce a 
	   patched version of the program, or to do a manual disassembly. As such,
	   a perfect duplicate of the original executable is stored as an image in
	   the .bdb; all address objects will consist of an offset into this image
	   and a size attribute to be used for reading data.
	2. Since a section is the only distinct block of code available at this
	   point, the start of each section is used to break the original single 
	   ADDRESS record up into smaller, component ADDRESS objects. Subsequent 
	   disassembly will use functions, structures, code, and data objects to
	   create further ADDRESS objects.

================================================================================
 Command Line Parameters
 
 bastard [-q] [-i FD] [-o FD] [-p str] [filename]
	-i [file]	-- use [file] instead of STDIN
	-o [file]	-- use [file] instead of STDOUT
	-e [file]	-- use [file] instead of STDERR
	-s [file]   -- run SeerC script [file] on target, then exit [disabled]
	-p [prompt] -- use [prompt] for interpreter prompt, e.g. "" for no prompt.
	-q			   -- quiet [suppress unnecessary output]
	-a          -- annoy user ["do you want to save?", etc]
	-c          -- colorize output
	-d          -- use debugging code [disabled]
	-l          -- use libopcodes [disabled]
	--          -- take code to assemble from STDIN [disabled]
	
	filename	-- file or database to open

================================================================================
 Using the Bastard CLI

 --Readline line-editor commands--
 --'Native' commands and aliases--
 --Dealing with the database--
	`db dump address | grep 804089AC`
	`db dump code | grep 804089AC`
 
 --Dealing with the disassembler--
 	Environment Variables:
	  PAGINATE
	  COLOR_TTY
	  QUIET
	  ANNOY_USER
	  DISABLE_EXEC
	  DEBUG_LIB
	`d | grep 0x804089AC`
	`d | more` /* note `more` does a SIGINT when you use 'q': the bastard 
	              does not like this */
	
 --The Instruction Stack--
 The "Instruction Stack" appears when DisassembleForward is called, unless the
 QUIET flag is set. It prints a character to STDOUT for each recursion; i.e.,
 when following a program jump or subroutine call. The characters change during
 the course of disassembly to give an animation that is at once neither useful
 nor entertaining. The characters are as follows:

   "+" = enter new stack level
   "|" = Step 1: Disassemble current instruction 
   "/" = Step 2: Analyze current instruction
   "-" = Step 3: Follow branch of execution [if applicable]
   "\" = Step 4: Advance to the next instruction

 Each iteration of DisassembleForward cleans its character from the screen 
 using a backspace character; this represents a successful disassembly.

 --Single-line seer scripts--
 --Multi-line seer scripts--

================================================================================
 The Disassembled Listing

 Notational Conventions:
 <08040000[x]        -- This address references or is referenced by another.
                        Legend:
								 < or >  : Reference is to (<) or from (>) this address
								 08040000: The rva referencing/referenced by this one
								 [x|r|w] : Type of reference: execute, read, or write
 (8040000 was 1233)  -- The relative offset 1233 was converted to the rva
                        8040000
 str_*:					-- This address is the start of a string.
 StrRef:             -- Operand is a reference to a string

================================================================================
 Interfacing With The Bastard


================================================================================
 Scripting The Bastard

 [basically a discussion of seer and such]


================================================================================
 The Bastard API

 For now, this is pulled directly from api.h :


/* Contents:_________________________________________________________________
 1.  Low-Level DB Interface
 2.  Generic Object Access
 3.  Low-Level Disassembler Access
 4.  Loading
 5.  Saving
 6.  Sections
 7.  Disassembly
 8.  Addresses
 9.  Names & Comments
 10. Structures & Constants
 11. Code
 12. Strings
 13. Imports
 14. Exports
 15. XRefs
 16. Functions
 17. Macros
 18. Search
 19. System Interaction                                                  */

/* Notes:_________________________________________________________________
 * + The following naming conventions are used:
 *       rva       Relative Virtual Address -- the address of a location of
 *                   code or data once the program has been loaded by the OS
 *       pa        Physical Address -- the offset of a location of code or
 *                   data within the executable file/image.
 *
 * + Most routines return 0 on error or failure; the reason for the error can
 *   be found by calling GetLastError();
 *   
 * + Routines that take character arrays as arguments generally do not require
 *   the length of the array. This because the strings in the DB are of
 *   fixed length [see bdb.h], and thus it simple enough for the caller to
 *   know the size of array they should use. Naturally this goes against all
 *   sane security practice; however anyone incorporating a program such as
 *   the bastard into their firewalling policy is pretty much beyond saving.
 *   Be warned: you are expected to know what you are doing.
 */

/* DB Interface -- db.c */
/* These are located in db.c, but are included here for the convenience of
 * the script writers who shouldn't really be using them, but probably will
 * regardless */

/* DB State -- a way to keep the DB from changing the current record inside
 *             a subroutine. Wrap all calls to the DB as follows:
 *                int *state;
 *                state = DBSaveState();
 *                 ...
 *                DBRestoreState(state);
 *             and you should be alright. */
void* DBSaveState();
int   DBRestoreState(void *state);
/* ---- Low-level DB access ---- */
/*   These expect *dest to be pre-allocated */
int DBIndexFirst(int index, void *dest);
int DBIndexNext(int index, void *dest);
int DBIndexPrev(int index, void *dest);
int DBIndexLast(int index, void *dest);
int DBIndexFind(int index, void *value, void *dest);
int DBTableFirst(int table, void *dest);
int DBTableNext(int table, void *dest);
int DBTablePrev(int table, void *dest);
int DBTableLast(int table, void *dest);
int DBRecordInsert(int table, void *src);
int DBRecordUpdate(int index, void *value, void *src);
int DBRecordDelete(int index, void *value);
int DBFindClosestPrev(int index, void *value, void *dest);
int DBFindClosestNext(int index, void *value, void *dest);
/* ---- generic 'object' access ---- */
/*   These calloc a struct and fill it; it is up to the caller to free */
struct address * GetAddressObject(long rva);
struct code * GetCodeObject(long rva);
struct section * GetSectionObject(char *name);
struct xref * GetXrefObject(int id);
struct int_code * GetIntCodeObject(int id);
struct fin_code * GetFinCodeObject(int id);
struct export * GetExportObject(long rva);
struct import * GetImportObject(long rva);
struct string * GetStringObject(long rva);
struct name * GetNameObject(long rva);
struct comment * GetCommentObject(int id);
struct function * GetFunctionObject(int id);
struct func_param * GetFuncParamObject(int id);
struct func_local * GetFuncLocalObject(int id);
struct f_inline * GetFInlineObject(int id);
struct inline_type * GetInlineTypeObject(int id);
struct structure * GetStructureTypeObject(int id);
struct struct_member * GetStructMemberObject(int id);
struct data_type * GetDataTypeObject(int id);
struct constant * GetConstantObject(int id);
struct seer_macro * GetSeerMacroObject(int id);

/* ---- Low level Disassembler access ---- */
/* These all call functions internal to the processor modules */
int DisPrologue(struct code **c);
int DisEpilogue(struct code **c);
int DisByteOrder();
int DisAddrSize();
int DisByteSize();
int DisWordSize();
int DisDwordSize();
char* DisGetSP();
char* DisGetIP();


/* Loading -- api_loadsave.c */
/*  LoadTarget : Open target file for disassenbly
 * 	Return Value:	1 -- Success 		0 -- Failure
 *  	Arguments:		char *target -- full name/path of file to open */
int LoadTarget( char *target );
/* LoadBDB : Open previously disassembled file 
 * 	Return Value:	1 -- Success 		0 -- Failure
 * 	Arguments:		char *dbname -- full name/path of .bdb file */
int LoadBDB( char *dbname );
/* SetFileFormat : Specify Format of Target [e.g. "ELF"]
 * 	Return Value:	1 -- Success 		0 -- Failure
 * 	Arguments:		char *file_format -- name of format
 * 	Notes:		Valid formats can be listed with GetValidFormats(); */
int SetFileFormat( char *file_format );
/* SetFileArch : Specify Processor Target was compiled for [e.g. "i386"]
 * 	Return Value:	1 -- Success 		0 -- Failure
 * 	Arguments:		char *file_arch -- name of architecture
 * 	Notes:		Valid architectures are listed with GetValidArchs(); */
int SetFileArch( char *file_arch );
/* SetOutputAssembler : Set the assembly language output format -- these are
 *                      .so's in $BASTARD_HOME/asm . The output assembler
 *                      determines the [default] formatting for screen and
 *                      .asm file output [e.g. AT&T vs Intel syntax]
 * 	Return Value:	1 -- Success 		0 -- Failure
 * 	Arguments:		char *asm_output -- name of target assembler */
int SetOutputAssembler( char *asm_output );
/* SetTargetLanguage : Specify the language the target was written in [e.g.
 *                     C, FORTRAN] ... this determines defaults for string
 *                     style, calling convention, etc. Language modules are
 *                     .so's in $BASTARD_HOME/lang
 * 	Return Value:	1 -- Success 		0 -- Failure
 * 	Arguments:		char *language -- name of language */
int SetTargetLanguage( char *language );

/* Saving -- api_loadsave.c  */
/* SaveDB : Save current DB as .bdb file
 * 	Notes : Uses ./$target.bdb for filename */
void SaveDB( );
/* SaveDBBak : Save current DB to a unique 'temp' filename
 * 	Notes: Uses ./$target.$timestamp.bdb as filename */
void SaveDBBak( );
/* CloseDB : Close the current DB and target [i.e. before loading a new one] */
void CloseDB( );
/* SaveDBAs : Save DB as the specified filename
 * 	Arguments:		char *filename -- name of file to save DB as */
void SaveDBAs( char *filename );
/* SaveAsASM : Save a disassembled listing of the file
 * 	Return Value :	0 -- Success 		-1 -- Failure
 * 	Arguments :		char *filename -- name of file to save listing as 
 * 	Notes : 		Will change to SaveAsLST and make new routine for 
 * 					GAS and NASM asm files */
int SaveAsASM( char *filename );
/* SaveAsDump : Save a  hex dump of the target
 * 	Return Value :	1 -- Success 		0 -- Failure
 * 	Arguments :		char *filename -- name of file to save dump as  */
int SaveAsDump( char *filename );
/* SaveAsDiff : Create a binary diff [patch] between the original target and
 *              the current DB image of the target. Useless if you have not
 *              patched the target inside the disassembler.
 * 	Return Value :	1 -- Success 		0 -- Failure
 * 	Arguments :		char *filename -- name of file to save diff as  */
int SaveAsDiff( char *filename );
/* SaveAsBinary : Create a new binary image of the target, using the DB image
 * 					of the target. Used to 'recreate' a missing or patched 
 * 					target.
 * 	Return Value :	1 -- Success 		0 -- Failure
 * 	Arguments :		char *filename -- name of file to save target as  */
int SaveAsBinary( char *filename );

/* Sections -- api_section.c */
 /* SECTION types and permissions */
/*  CreateSection :  Create a program section or segment
 * 	Return Value:	1 = success, 0 = error
 *  	Arguments:		char *name -- name of section [must be unique]
 *  	               long start -- starting rva of section 
 *  	               long pa    -- starting physical addr of section
 *  	               int  size  -- size of section, in bytes
 *  	               int  type  -- section type ORed with permissions : 
	types : SECTION_HDR	SECTION_CODE SECTION_DATA SECTION_RSRC SECTION_RELOC
	        SECTION_IMPORT SECTION_EXPORT SECTION_DEBUG SECTION_CMT	
	perms : SECTION_EXECUTE SECTION_WRITE	SECTION_READ           */
int CreateSection( char *name, long start, int size, long pa, int type );
/*  DeleteSection : Delete a program section [be careful!]
 * 	Return Value:	1 = success, 0 = error
 *  	Arguments:		char *name -- name of section to delete  */
int DeleteSection( char *name );
/*  GetSection : Finds the Section that contains a given addr
 *    Return Value: 1 if found, 0 if not
 *    Arguments:    long rva   -- known address
 *                  section *s -- empty structure to fill w/ found section */
int GetSection( long rva, struct section *s);
/*  GetSectionStart : Get starting address of a given section
 * 	Return Value: rva of start of section, 0 if not found	
 *  	Arguments:	  char *name -- name of section to get the start of	 */
 	/* FIX: should increase the end addr of the prev section thru curr->end */
long GetSectionStart( char *name );
/*  GetSectionEnd : Get address of the last byte in a given section
 * 	Return Value: rva of end of section [not last ADDRESS in section!] or 0	
 *  	Arguments:	  char *name -- name of section to get the end of	 */
long GetSectionEnd( char *name );
/*  GetSectionName : Get name of section
 * 	Return Value:	1 if found, 0 if not
 *  	Arguments:		long rva     -- address within section 
 *  	               char buf[32] -- buffer to fill with section name */
int  GetSectionName( long rva, char* buf );
/*  GetSectionFlags : Get section type/permissions 
 * 	Return Value:	Type and permissions OR'ed together
 *  	Arguments:		char *name -- name of section to get the flags of */
int  GetSectionFlags( char *name );
/*  SetSectionStart :  Set the starting address of a section
 * 	Return Value:	  1 if success, 0 if error
 *  	Arguments:		  char *name -- name of section
 *  	                 long start -- new starting rva of section */
int  SetSectionStart( char *name, long start );
/*  SetSectionEnd : Set the ending address of a section
 * 	Return Value:	1 if success, 0 if error
 *  	Arguments:		char *name -- name of section
 *  	               long end   -- rva of the "new" last byte in the section */
int  SetSectionEnd( char *name, long end );
/*  SetSectionName : Set the name of a section
 * 	Return Value:	1 if success, 0 if error
 *  	Arguments:		char *name == old name, char *new_name = new name */
int  SetSectionName( char *name, char *new_name );
/*  SetSectionFlags : Change the type/permissions of a section
 * 	Return Value:	1 id success, 0 if failure
 *  	Arguments:		char *name -- name of section
 *  	               int  flags -- new type and permissions ORed together */
int  SetSectionFlags( char *name, int flags );

/* Disassembly -- api_disasm.c */
/*  CreateHeader : Create ASCII representation of the file/program header
 * 	Return Value:	1 id success , 0 if failure
 *  	Arguments:	   char *header = ASCII representation of header	 */
int CreateHeader( char *header );
/*  PrintHeader : Print ASCII representation of header to stdout
 * 	Return Value: none	
 *  	Arguments:	  none [uses header in target.header]	 */
void PrintHeader( );
/*  InitDisassembler : Load and initialize disassembler module
 *                     This sets the processor type for subsequent disassembly.
 *                     Note that the disasm_eng structure will be passed to
 *                     the processor module in order to perform any cpu-
 *                     specific init [like default address size] -- so do all
 *                     mods to that structure before making this call.
 * 	Return Value:	1 if success, 0 if failure
 *  	Arguments:		char *disasm -- name of processor module to load. This is
 *  	                               the part of the name between "lib" and 
 *  	                               ".so", e.g. libi386.so would be loaded as
 *  	                               "i386" */
int InitDisassembler( char *disasm );
/*  DisassembleFull : Perform a full-featured deluxe-o-rama disassembly:
 *                     1. DisassembleForward() from each known function
 *                     2. DisassembleForward() from the entry point
 *                     3. Perform all passes under $BASTARD_HOME/engines
 * 	Return Value:	1 if success, 0 if error
 *  	Arguments:		none  */
int DisassembleFull( );
/*  DisassembleDumb : Perform a simple, brain-dead disassembly:
 *                     1. Do a DisassembleSection() of each section of type
 *                        SECTION_CODE or with SECTION_EXECUTE permissions
 *                     2. That's it.
 * 	Return Value:	1 if success, 0 if failure
 *  	Arguments:		none  */
int DisassembleDumb( );
/*  DisassembleSection : Perform a linear disassembly of a section. This does
 *                       no flow-of execution; rather it calls DisassembleRange
 *                       for the starting rva & size of the section
 * 	Return Value:	1 if success, 0 if failure
 *  	Arguments:		char *name -- name of section to disassemble */
int DisassembleSection( char *name );
/*  DisassembleRange :   Perform a linear disassembly on a range of addresses;
 *                       this blindly disassembles instruction after instruction
 *                       with no branch following/etc, until the end of the
 *                       range is reached.
 * 	Return Value:	 1 if success, 0 if failure
 *  	Arguments:		 long rva  -- address to start disassembly at
 *  	                int  size -- size of range to disassemble [in bytes] */
int DisassembleRange( long rva, int size );
/*  DisassembleForward : Perform a flow-of-execution disassembly starting at
 *                       the given address. The disassembly attempts to 
 *                       follow branches and will not overwrite existing CODE
 *                       objects [they will be skipped -- as with the other
 *                       disassembly routines. this behavior is determined in
 *                       DisAddr(), the backend to all of these api routines].
 * 	Return Value:	
 *  	Arguments:		 */
int DisassembleForward( long rva );
/*  DisassemblePass :    Performa a single post-disassembly pass on the target.
 *                       Passes are in the pass#.sc files in the engines/
 *                       directory; the # in the file name is the number used
 *                       to specify which pass to perform.
 * 	Return Value:	1 if success, 0 if failure
 *  	Arguments:		int pass -- the number of the pass to perform */
int DisassemblePass( int pass );
/*  DisassembleAllPasses : Performs all passes on the target 
 * 	Return Value:	1 if success, 0 if failure
 *  	Arguments:		none */
int DisassembleAllPasses();

/* Addresses -- api_address.c */
/*  IsValidAddr :  Determine if given address is valid [ this is interpreted
 *                 to mean 'the given rva exists in some section', not 'the
 *                 given rva belongs to an ADDRESS object]. This is a bit
 *                 redundant with GetSection(), but makes code more clear.
 * 	Return Value:	1 if valid, 0 if invalid
 *  	Arguments:		long rva -- address to be checked */
int IsValidAddr( long rva );
/*  IsValidCodeAddr : Determine if given address is a valid code address 
 *                    [i.e., does it exist in a section of type SECTION_CODE]
 * 	Return Value:	1 if valid, 0 if invalid
 *  	Arguments:		long rva -- address to be checked */
int IsValidCodeAddr( long rva );
/*  AddrExists : Determine if the given address is an ADDRESS object
 * 	Return Value:	1 if it exists, 0 if it does not
 *  	Arguments:		long rva -- address to be checked */
int AddrExists( long rva );
/*  FindClosestAddr : Find ADDRESS object containing given address --
 *                    this searches if 'rva' exists and, if it doesn't,
 *                    searches backwards looking for the ADDRESS object
 *                    that begins before rva.
 * 	Return Value:	 the rva closest to the given address
 *  	Arguments:		 long rva -- address to be located */
long FindClosestAddr( long rva );
/*  DefineAddr :      Create an ADDRESS object at the given address, of the
 *                    specified size. This will resize existing addresses if
 *                    need be.
 * 	Return Value:	 1 if success, 0 if failure
 *  	Arguments:		 long rva   -- address to be created
 *  	                int  size  -- size of address in bytes
 *  	                long  pa   -- physical address of 'rva'
 *  	                int  flags -- Flags for ADDRESS object, e.g. : 
 *                                   ADDR_CODE ADDR_DATA */
int DefineAddr( long rva, int size, long pa, int flags );
/*  UndefineAddr :  Remove ADDRESS object for given address
 * 	Return Value:	1 if success, 0 if failure
 *  	Arguments:		long rva -- address to delete */
int UndefineAddr( long rva );
/*  asmsprintf : print formatted line of assembly to buffer. This is like
 *                 sprintf(), only with custom format specificers:
 *                  %a - rva            %p - pa               %n - name  
 *                  %m - mnemonic       %b - hex bytes        %S - section name
 *                  %d - dest operand   %s - source operand   %t - aux operand
 *                  %c - comment        %x - xrefs
 *                 These specifiers allow a length byte, which for string
 *                 specifiers [%a %pa %n %m %s %d %x %S %c] limits the length
 *                 of the string, for xrefs or hex bytes [%x %b] soecifies 
 *                 how many to display. Examples:
 *                     %4x  - show up to 4 xrefs
 *                     %8b  - show up to 8 hexadecimal bytes
 *                     %64c - show up to 64 characters of comment
 *                 There are conditional punctuation specifiers:
 *                  %, - insert a comma if the next specifier is non-null
 *                  %; - insert a semicolon if any text follows
 *                  %: - append a colon if any text precedes it
 *                  %^ - append a newline if any text precedes it
 *                 The default format string for the Intel architecture is:
 *                     "%n%:%^%a %8b\t%m\t%d%, %s %;%c %x"
 *                 Note that as with sprintf(), a '%' can be printed by using
 *                 "%%".
 * 	Return Value:	1 if success, 0 if failure
 *  	Arguments:		char *buf    -- buffer to fill. It better e big enough.
 *  	               char *format -- format string 
 *  	               struct address *addr -- address to be printed */
int asmsprintf(char *buf, char *format, struct address *addr);
/*  : 
 * 	Return Value:	
 *  	Arguments:		 */
int PrintAddress( long rva );
/*  MakeCode : 
 * 	Return Value:	
 *  	Arguments:		 */
int MakeCode( long addr );
/*  MakeData : 
 * 	Return Value:	
 *  	Arguments:		 */
int MakeData( long addr );
/*  GetPA : 
 * 	Return Value:	
 *  	Arguments:		 */
long GetPA( long rva );
/*  GetRVA : 
 * 	Return Value:	
 *  	Arguments:		 */
long GetRVA( long pa );
/*  GetAddrType : 
 * 	Return Value:	
 *  	Arguments:		 */
int GetAddrType( long rva );
/*  GetAddrSize : 
 * 	Return Value:	
 *  	Arguments:		 */
int GetAddrSize( long rva );
/*  GetFlags : 
 * 	Return Value:	
 *  	Arguments:		 */
int GetFlags( long addr );
/*  GetAddrBytes : 
 * 	Return Value:	
 *  	Arguments:		 */
int GetAddrBytes( long rva, char *buf );
/*  GetNextAddr : 
 * 	Return Value:	
 *  	Arguments:		 */
long GetNextAddr( long rva );
/*  GetPrevAddr : 
 * 	Return Value:	
 *  	Arguments:		 */
long GetPrevAddr( long rva );
/*  SetFlags : 
 * 	Return Value:	
 *  	Arguments:		 */
int SetFlags( long rva, int flags );


/* Names & Comments -- api_namecmt.c */
/*  SetName : 
 * 	Return Value:	
 *  	Arguments:		 */
int SetName( long rva, char* name, int type );
/*  GetName : 
 * 	Return Value:	
 *  	Arguments:		 */
int GetName( long rva, char* buffer );
/*  GetNameType: 
 * 	Return Value:	
 *  	Arguments:		 */
int GetNameType(long addr);
/*  GetAddrByName: 
 * 	Return Value:	
 *  	Arguments:		 */
long GetAddrByName( char *name );
/*  DefineAddrComment : 
 * 	Return Value:	
 *  	Arguments:		 */
int DefineAddrComment( long rva, char *buf );
/*  GetAddrComment : 
 * 	Return Value:	
 *  	Arguments:		 */
int GetAddrComment( long rva );
/*  SetAddrComment : 
 * 	Return Value:	
 *  	Arguments:		 */
int SetAddrComment( long rva, int id );
/*  GetComment: 
 * 	Return Value:	
 *  	Arguments:		 */
int GetComment( int id, char *buf );
/*  DefineComment: 
 * 	Return Value:	
 *  	Arguments:		 */
int DefineComment( char *buf, int type );
/*  SetComment: 
 * 	Return Value:	
 *  	Arguments:		 */
int SetComment( int id, char *buf );
/*  DelComment: 
 * 	Return Value:	
 *  	Arguments:		 */
int DelComment( int id ) ;



/* Structures & Constants -- api_structconst.c */
/*  GetAddrConstant : 
 * 	Return Value:	
 *  	Arguments:		 */
int GetAddrConstant( long rva );
/*  GetAddrStruct : 
 * 	Return Value:	
 *  	Arguments:		 */
int GetAddrStruct( long rva );
/*  SetAddrConstant : 
 * 	Return Value:	
 *  	Arguments:		 */
int SetAddrConstant( long rva, int id );
/*  SetAddrStruct : 
 * 	Return Value:	
 *  	Arguments:		 */
int SetAddrStruct( long rva, int id );
// *buf is buf[32]
/*  DefineConstant: 
 * 	Return Value:	
 *  	Arguments:		 */
int DefineConstant( char *name, int value);
/*  GetConstant: 
 * 	Return Value:	
 *  	Arguments:		 */
int GetConstant(int value, char buf[32]);
/*  GetConstantNext: 
 * 	Return Value:	
 *  	Arguments:		 */
int GetConstantNext( char buf[32]);
/*  GetConstantName: 
 * 	Return Value:	
 *  	Arguments:		 */
int GetConstantName( unsigned long id, char buf[32]);
/*  GetConstantID: 
 * 	Return Value:	
 *  	Arguments:		 */
unsigned long GetConstantID( char *name);
/*  DelConstant: 
 * 	Return Value:	
 *  	Arguments:		 */
int DelConstant( unsigned long id);


/* Code -- api_code.c  */
/*  DefineCode: 
 * 	Return Value:	
 *  	Arguments:		 */
int DefineCode(long rva, struct code *c);
/*  UndefineCode: 
 * 	Return Value:	
 *  	Arguments:		 */
int UndefineCode(long rva);
/*  GetNextCode: 
 * 	Return Value:	
 *  	Arguments:		 */
long GetNextCode( long rva );
/*  GetNextData: 
 * 	Return Value:	
 *  	Arguments:		 */
long GetNextData( long rva );
/*  GetCodeRep: 
 * 	Return Value:	
 *  	Arguments:		 */
int GetCodeRep( char *buf, char *fmt, long rva ) ;
/*  GetMnemonic: 
 * 	Return Value:	
 *  	Arguments:		 */
int GetMnemonic( long rva, char *buf );
/*  GetSrc: 
 * 	Return Value:	
 *  	Arguments:		 */
int GetSrc( long rva, char *buf );
/*  GetDest: 
 * 	Return Value:	
 *  	Arguments:		 */
int GetDest( long rva, char *buf );
/*  GetAux: 
 * 	Return Value:	
 *  	Arguments:		 */
int GetAux( long rva, char *buf );
/*  GetMnemType: 
 * 	Return Value:	
 *  	Arguments:		 */
int GetMnemType( long rva );
/*  GetSrcType: 
 * 	Return Value:	
 *  	Arguments:		 */
int GetSrcType( long rva );
/*  GetDestType: 
 * 	Return Value:	
 *  	Arguments:		 */
int GetDestType( long rva );
/*  GetAuxType: 
 * 	Return Value:	
 *  	Arguments:		 */
int GetAuxType( long rva );

/*	Strings -- api_strings.c */
/*  GetString: 
 * 	Return Value:	
 *  	Arguments:		 */
int GetString( long rva, char *buf, int len );
/* gets entire string from binary file, puts it in buf, returns length */
int GetEntireString(long rva, char **buf);
/*  DefineString: 
 * 	Return Value:	
 *  	Arguments:		 */
int DefineString( long rva );
/*  DumpStrings: 
 * 	Return Value:   */	
int DumpStrings( ); /* print all strings/addresses to stdout */
/* rva : start of range   size: extent of range  type: string type */
int FindStringsInRange(long rva, int size, int type);


/* Imports -- api_importexport.c */
/*  DefineImport: 
 * 	Return Value:	
 *  	Arguments:		 */
int DefineImport( long rva, char *lib, char* name, int type );
/*  GetImportName: 
 * 	Return Value:	
 *  	Arguments:		 */
int GetImportName( long rva, char *buf );
/*  GetImportLib: 
 * 	Return Value:	
 *  	Arguments:		 */
int GetImportLib( long rva, char *buf );
/*  GetImportType: 
 * 	Return Value:	
 *  	Arguments:		 */
int GetImportType( long rva );
/*  DumpImports: 
 * 	Return Value:	
 *  	Arguments:		 */
int DumpImports( );
/*  DefineLibrary: 
 * 	Return Value:	
 *  	Arguments:		 */
int DefineLibrary( char *name, int v_hi, int v_lo );
/*  DumpLibs: 
 * 	Return Value:	
 *  	Arguments:		 */
int DumpLibs( );	/* print all libraries to stdout */


/* Exports -- api_importexport.c */
/*  DefineExport: 
 * 	Return Value:	
 *  	Arguments:		 */
int DefineExport( long rva, char *name );
/*  GetExport: 
 * 	Return Value:	
 *  	Arguments:		 */
int GetExport( char *name );
/*  DumpExports: 
 * 	Return Value:	
 *  	Arguments:		 */
int DumpExports( ); /* list all entry points */


/* XRefs -- api_xref.c */
/*  GetRefTo: 
 * 	Return Value: from_rva or 0 if no matches	
 *  	Arguments:	  rva  -- address to find references to
 *  	          	  type -- type of xref to match [OP_R | OP_W | OP_X] or 0 */
long GetRefTo( long rva, int type ) ; 		//get first
/*  GetNextRefTo: 
 * 	Return Value:	from_rva or 0 if no more matches	
 *  	Arguments:	  type -- type of xref to match [OP_R | OP_W | OP_X] or 0 */
long GetNextRefTo( int type ) ; //get array of structs; return num
/*  GetRefFrom: 
 * 	Return Value: to_rva or 0 if no matches	
 *  	Arguments:	  rva  -- address to find references from
 *  	          	  type -- type of xref to match [OP_R | OP_W | OP_X] or 0 */
long GetRefFrom( long rva, int type ) ;
/*  GetNextRefFrom: 
 * 	Return Value: to_rva or 0 if no more matches	
 *  	Arguments:	  type -- type of xref to match [OP_R | OP_W | OP_X] or 0 */
long GetNextRefFrom( int type ) ;
/*  DefineRef: 
 * 	Return Value: 1 on success, 0 on error 
 *  	Arguments:		 */
int DefineRef( long from, long to, int type );


/* Functions -- api_functions.c */
/*  DefineFunc: 
 * 	Return Value:	
 *  	Arguments:		 */
int DefineFunc( long rva, char * name, int size );
/*  GetFuncName: 
 * 	Return Value:	
 *  	Arguments:		 */
int GetFuncName( long rva, char *buf );
/*  GetFuncStart: 
 * 	Return Value:	
 *  	Arguments:		 */
int GetFuncStart( long rva );
/*  GetFuncEnd: 
 * 	Return Value:	
 *  	Arguments:		 */
int GetFuncEnd( long rva );
/*  GetFuncByName: 
 * 	Return Value:	
 *  	Arguments:		 */
int GetFuncByName( char *name );
/*  SetFuncName: 
 * 	Return Value:	
 *  	Arguments:		 */
int SetFuncName( long rva, char *name );
/*  SetFuncSize: 
 * 	Return Value:	
 *  	Arguments:		 */
int SetFuncSize( long rva, int size );


/* Macros -- api_macros.c */
	/* Macro Types */
#define SEERC_CODE	0x01
#define SEERC_FILE	0x02
/*  RunScript: Run a script contained in a file 
 * 	Return Value:	Exit status of script
 *  	Arguments:		char *name -- name of file to compile and execute */
int RunScript( char *name );
/*  RunScriptText: Compile and run the provided code
 * 	Return Value:	Exit status of script
 *  	Arguments:		char *script -- code to compile and execute */
int RunScriptText( char *script );
/*  RunCompiledScript: Run a precompiled script
 * 	Return Value:	  Exit status of script
 *  	Arguments:		  char *name -- name of file to load and run */
int RunCompiledScript(char *name);
/*  CompileScript: Compile a script file
 * 	Return Value:	1 if success, 0 if failure
 *  	Arguments:		char *name -- name of file to compile */
int CompileScript(char *name);
/*  DefineMacro: 
 * 	Return Value:	
 *  	Arguments:		 */
int DefineMacro( char *name, char *macro );
/*  UndefineMacro: 
 * 	Return Value:	
 *  	Arguments:		 */
int UndefineMacro( char *name );
/*  RunMacro: 
 * 	Return Value:	
 *  	Arguments:		 */
int RunMacro( char *name );
/*  RecordMacro: 
 * 	Return Value:	
 *  	Arguments:		 */
int RecordMacro( char *name );

/* Search -- api_search.c */
/* Each of these 'Find' functions initiates a search if *search is NULL --
 * the Find becomes a FindFirst, and *search is filled with a search handle.
 * Subsequent calls to the Find function act as a FindNext, until the search
 * returns 0 hits [and the handle is automatically freed]. If you are finished
 * with the search before it returns 0 --for example if you only want the first
 * ten matches -- then you must call FreeSearchHandle to dealloc the memory
 * used for *search. */

/*  FindBytes: Find a sequence of bytes in the target image
 * 	Return Value:	Rva of found location or 0 if not found
 *  	Arguments:		char * bytes -- sequence of bytes to search for
 *  	               int    len   -- length of the sequence 
 *  	               int **search -- void pointer to store search handle in */
int FindBytes( unsigned char *bytes, int len, int **search);
/*  FindOperand:     Find an operand in all the code objects
 * 	Return Value:	rva of match or 0 if no match
 *  	Arguments:		char *str    -- operand to search for 
 *  	               int **search -- void pointer to store search handle in */
int FindOperand( char *str, int **search);
/*  FindMnemonic:    Find a mnemonic in all the code objects
 * 	Return Value:	rva of match or 0 if no match
 *  	Arguments:		char *str    -- mnemonic to search for
 *  	               int **search -- void pointer to store search handle in */
int FindMnemonic( char *str, int **search);
/*  FindConstant:    Find an Imeediate Value/Constant in all the CODE objects
 * 	Return Value:	rva of match or 0 if no match
 *  	Arguments:		long value   -- value to search for 
 *  	               int **search -- void pointer to store search handle in */
int FindConstant( long value, int **search);
/*  FreeSearchHandle: Use this if you terminate a search before viewing all
 *                    the hits.
 * 	Return Value:	 Returns a count of the matches examined
 *  	Arguments:		 int **search -- void pointer to a search handle */
int FreeSearchHandle( int **search );

/* The following searches are one-offs and do not use a search handle.
 * These take a given operand [assumed to be a regiser or address] and
 * search backwards or forwards for up to 'ttl' instructions looking for
 * an instruction where that operand is accessed with OP_R, OP_W, or OP_X
 * permissions. 
 *    Return Value:   rva where the match occurs or 0 if ttl expired
 *    Arguments:      char *operand -- operand to search for
 *                    long rva      -- address to start searching after/before
 *                    int ttl       -- max # of addresses to search */
long FindNextOpRead( char *operand, long rva, int ttl );
long FindPrevOpRead( char *operand, long rva, int ttl );
long FindNextOpWrite( char *operand, long rva, int ttl );
long FindPrevOpWrite( char *operand, long rva, int ttl );
long FindNextOpExec( char *operand, long rva, int ttl );
long FindPrevOpExec( char *operand, long rva, int ttl );

/* System Interaction -- api_system.c */
/*  Message : Print a message to STDOUT unless QUIET flags is set 
 *  	Arguments:	char *str -- Message to print	 */
void Message( const char *str ) ;
/*  ErrorMsg: Print Error Code & Message w/newline, regardless of QUIET flag
 *  	Arguments:	int num -- Error Code    char *str -- Message to print	 */
void ErrorMsg( int num, const char *str ) ;
/*  Quit: Exit the bastard */
void Quit( ) ;
/*  Exec : Execute the target via system() 
 * 	Return Value:	exit status of process
 *  	Arguments:		char *args -- arguments to append to command line */
int Exec( char *args );
/*  Debug: Execute target in gdb via system()
 * 	Return Value:	exit status of process */
int Debug( );
/*  Core: Execute target with core_dumps enabled; allow user to coredump the
 *        target with a hotkey combination
 * 	Return Value:	1 = success   0 = failure */
int Core( ); 
/*  Panic: Get a snapshot of the kernel as the target is running -- basically
 *         dump system state as you would in a kernel panic
 * 	Return Value:	1 = success   0 = failure */
int Panic( ); 
/*  GetValidArchs: Dump a list of all supported processors to STDOUT 
 * 	Return Value:	0 */
int GetValidArchs( );
/*  GetValidFormats: Dump a list of all supported file formtas to STDOUT 
 * 	Return Value:	0 */
int GetValidFormats( );
/*  GetEnvFlag: Test whether environment flag is true. Flags:
 *                     DB_LOADED TGT_MAPPED DB_MOD TGT MOD PASS_1 PASS_2
 *                     CODE_INT CODE_FIN PAGINATE COLOR_TTY QUIET ANNOY_USER
 *                     DISABLE_EXEC DEBUG_LIB
 * 	Return Value:	1 (set) or 0 (not set)
 *  	Arguments:		int flag -- one of the above flags  */
int GetEnvFlag( int flag );
/*  SetEnvFlag: Set an environment flag 
 * 	Return Value:	Returns 0 always
 *  	Arguments:		int flag -- flag to set */
int SetEnvFlag( int flag );
/*  ClearEnvFlag: Un-set an environment flag 
 * 	Return Value:	Returns 0 always
 *  	Arguments:		int flag -- flag to clear */
int ClearEnvFlag( int flag );
/*  SetTtyAsmFmt: Set asmsprintf format string for CODE on a tty
 *  	Arguments:   char *fmt -- new default format string */
void SetTtyAsmFmt( char *fmt );
/*  SetTtyDataFmt: asmsprintf format string for DATA on a tty 
 *  	Arguments:   char *fmt -- new default format string */
void SetTtyDataFmt( char *fmt );
/*  SetFileAsmFmt: asmsprintf format string for CODE to a file
 *  	Arguments:   char *fmt -- new default format string */
void SetFileAsmFmt( char *fmt );
/*  SetFileDataFmt: asmsprintf format string for DATA to a file
 *  	Arguments:   char *fmt -- new default format string */
void SetFileDataFmt( char *fmt );
/*  SetLprAsmFmt: asmsprintf format string for CODE on a printer
 *  	Arguments:   char *fmt -- new default format string */
void SetLprAsmFmt( char *fmt );
/*  SetLprDataFmt: asmsprintf format string for DATA on a printer
 *  	Arguments:   char *fmt -- new default format string */
void SetLprDataFmt( char *fmt );
/*  SetLastError:  Sets the 'last error' internal variable 
 * 	Return Value:	Returns 0 always
 *  	Arguments:		int errnum -- error code for last error */
int SetLastError( int errnum );
/*  GetLastError: Returns error code of last error
 * 	Return Value:	Error Code or 0 if failure
 *  	Arguments:		none */
int GetLastError( );
/*  GetLastErrorString: Returns pointer to string representing the last error
 * 	Return Value:	   const char * -- pointer to string or NULL if failure
 *  	Arguments:		   none */
char* GetLastErrorString( );
/*  PrintErrorMsg:  Print string for error code to STDOUT
 *  	Arguments:	  int errnum -- error code to print	 */
void PrintErrorMsg(int errnum);
/*  SprintErrorMsg:  Fills buffer with string for error code
 *  	Arguments:		char buf[80] -- buffer for string
 *  	               int  errnum  -- error code to look up */
void SprintErrorMsg(char *buf, int errnum);



 
================================================================================
 Database Structure


   SECTION     -- Distinct regions of code or data within the program
	               Indexed by name and rva
		char *name[ ],                -- name of SECTION
		ulong rva,                    -- starting ADDRESS of SECTION
		int flags,                    -- SECTION type and permissions
		ulong size                    -- number of bytes in SECTION

		Section Flags:
			SECTION_EXECUTE 	0x01
			SECTION_WRITE   	0x02
			SECTION_READ 		0x04
			SECTION_HDR     	0x1000
			SECTION_CODE 		0x1100
			SECTION_DATA 		0x1200
			SECTION_RSRC 		0x1300
			SECTION_IMPORT  	0x1400
			SECTION_EXPORT  	0x1500
			SECTION_DEBUG   	0x1600
			SECTION_CMT     	0x1700
			SECTION_RELOC   	0x1800

   ADDRESS     -- Distinct code or data units within the program
	               Indexed by rva and pa
		ulong rva,                    -- relative virtual address
		ulong pa,                     -- physical address
		ushort size,                  -- number of bytes in ADDRESS
		int flags,                    -- type of ADDRESS [code, data...] 
		ulong dataConst,              -- id of CONSTANT associated with ADDRESS
		ulong structure,              -- id of STRUCTURE associated with ADDRESS
		ulong comment                 -- id of COMMENT associated with ADDRESS
		
		Address Flags:
			ADDR_CODE    		0x01
			ADDR_DATA    		0x02
			ADDR_IMPORT     	0x10
			ADDR_EXPORT     	0x20
			ADDR_SECTION 		0x40
			ADDR_STRUCT     	0x100
			ADDR_STRING     	0x200
			ADDR_FUNCTION   	0x300
			ADDR_INLINE     	0x400
			ADDR_NAME    		0x1000
			ADDR_COMMENT 		0x2000
			ADDR_SYMBOL  		0x4000

   XREF       -- References to code/data 
	              Indexed by id, from_rva, and to_rva
   	ulong id,                     -- unique number in xref sequence
		ulong from_rva,               -- rva the reference is to
		ulong to_rva,                 -- rva the reference is from
		int type                      -- type of reference

		Xref Types:
			XREF_READ    		0x01
			XREF_WRITE   		0x02
			XREF_EXEC    		0x04

   CODE        -- An address object interpreted as a machine instruction
	               Indexed by rva, dest, src, and mnemonic
		ulong rva,                    -- ADDRESS where this CODE is
		ushort flags,                 -- CODE type
		char mnemonic[],              -- text of mnemonic
		char dest[],                  -- text version of destination operand
		char src[],                   -- text version of source operand
		char aux[],                   -- text version of additional operand
		int mnemType,                 -- type pf mnemonic
		int destType,                 -- type of destination operand
		int srcType,                  -- type of source operand
		int auxType,                  -- type of additional operand
		ulong destConst,              -- id of CONSTANT for dest, if used
		ulong srcConst,               -- id of CONSTANT for src, if used
		ulong auxConst                -- id of CONSTANT for aux, if used

		Code Flags:
			/* Not yet used */
		Mnemonic Types:
			INS_BRANCH   0x01     /* Unconditional branch */
			INS_COND     0x02     /* Conditional branch */
			INS_SUB      0x04     /* Jump to subroutine */
			INS_RET      0x08     /* Return from subroutine */
			INS_ARITH    0x10     /* Arithmetic inst */
			INS_LOGIC    0x20     /* logical inst */
			INS_FPU      0x40     /* Floating Point inst */
			INS_FLAG     0x80     /* Modify flags */
			INS_MOVE     0x0100
			INS_ARRAY    0x0200   /* String and XLAT ops */
			INS_PTR      0x0400   /* Load EA/pointer */
			INS_STACK    0x1000   /* PUSH, POP, etc */
			INS_FRAME    0x2000   /* ENTER, LEAVE, etc */
			INS_SYSTEM   0x4000   /* CPUID, WBINVD, etc */
			INS_BYTE     0x10000  /* operand is  8 bits/1 byte  */
			INS_WORD     0x20000  /* operand is 16 bits/2 bytes */
			INS_DWORD    0x40000  /* operand is 32 bits/4 bytes */
			INS_QWORD    0x80000

		Operand Types:
			OP_R         0x001    /* operand is READ */
			OP_W         0x002    /* operand is WRITTEN */
			OP_X         0x004    /* operand is EXECUTED */
			OP_REG       0x100    /* register */
			OP_IMM       0x200    /* immediate value */
			OP_IND       0x300    /* indirect memory reference */
			OP_REL       0x400    /* relative Address [offset] */
			OP_BPTR      0x500    /* BYTE Pointer */
			OP_WPTR      0x600    /* WORD Pointer */
			OP_DPTR      0x700    /* DWORD Pointer */
			OP_STRPTR    0x800    /* STRING Pointer */
			OP_EXPR      0x900    /* Address Expression [e.g. SIB byte] */
			OP_UNK       0xA00    
			OP_SIGNED    0x1000   /* operand is signed */
			OP_UNSIGNED  0x2000   /* operand is unsigned */
			OP_CONST     0x4000   /* operans is a constant */
			OP_BYTE      0x10000  /* operand is  8 bits/1 byte  */
			OP_WORD      0x20000  /* operand is 16 bits/2 bytes */
			OP_DWORD     0x40000  /* operand is 32 bits/4 bytes */
			OP_QWORD     0x80000  /* operand is 64 bits/8 bytes */

	
	INT_CODE		-- Representation of CODE in an intermediate language
	               Indexed by id and rva
   	ulong id,                     -- unique number in sequence
		ulong rva,                    -- ADDRESS where this code starts
		int size,                     -- number of bytes this code represents
		char lvalue[],                -- L-Value [destination]
		char rvalue[]                 -- R-Value [expression]

	FIN_CODE 	-- Representation of INT_CODE in the final, high-level language
	               Indexed by id and rva
   	ulong id,                     -- unique number in sequence
		ulong rva,                    -- ADDRESS where this code starts
		int size,                     -- number of bytes this code represents
		char line[]                   -- final line of source code

   EXPORT      -- Program Entry Points
	               Indexed by rva
		ulong rva                     -- ADDRESS that is exported

   IMPORT      -- External code references in the program
	               Indexed by rva and library
		int library,                  -- id of LIBRARY owning this import
		int type,                     -- type of import
		ulong rva							-- ADDRESS of import [fn pointer]

		Import Types
			/* unused -- probably code, data */

	LIBRARY		-- Shared libraries required by the program
	               Indexed by id and name
   	ulong id,                     -- unique number in sequence
		char name[],                  -- name of library
		int ver_hi,                   -- high-order version number [#.]
		in ver_lo                     -- low-order version number [.#]
		
   STRING      -- Data regions considered strings
	               Indexed by rva
		ulong rva,                    -- ADDRESS where string starts
		int length,                   -- size of string in bytes
		char text[]                   -- first 255 bytes of string

   NAME        -- Unique text strings associated with address objects
	               Indexed by rva, name
		ulong rva,                    -- ADDRESS this NAME applies to
		int type,                     -- type of NAME
		char name[]                   -- NAME

		Name Types:
			NAME_AUTO    		0x01
			NAME_SUB     		0x02
			NAME_USER    		0x10
			NAME_SYMBOL  		0x20

   COMMENT     -- Text strings associated with address objects
	               Indexed by id
   	ulong id,                     -- unique number in sequence
		int type,                     -- type of COMMENT
		char text[]                   -- COMMENT

		Comment Types:
			CMT_AUTO     		0x01
			CMT_USER     		0x02

   FUNCTION    -- Code regions defined as a function
	               Indexed by id and rva
   	ulong id,                     -- unique number in sequence
		ulong rva,                    -- ADDRESS where function starts
		ushort size,                  -- size of function in bytes
		ulong comment                 -- id of COMMENT associated with FUNCTION

	FUNC_PARAM	-- Named parameters associated with a FUNCTION
	               Indexed by id and func
   	ulong id,                     -- unique number in sequence
		ulong func,                   -- id of FUNCTION associated with param
		ulong type,                   -- id of DATA_TYPE associated with param
		int offset,                   -- offset from stack frame
		char name[]                   -- name associated with this param

	FUNC_LOCAL	-- Local variables associated with a FUNCTION
	               Indexed by id and func
   	ulong id,                     -- unique number in sequence
		ulong func,                   -- id of FUNCTION associated with var
		ulong type,                   -- id of DATA_TYPE associated with var
		int offset,                   -- offset from stack frame
		char name[]                   -- name associated with this param

   F_INLINE    -- Code regions associated with C macro/inline function
	               Indexed by id and rva
   	ulong id,                     -- unique number in sequence
		ulong type                    -- id of INLINE_TYPE
	
	INLINE_TYPE -- Predefined 'template' types for F_INLINE
	               Indexed by id and rva
   	ulong id,                     -- unique number in sequence
		char name[],                  -- name of this type
		ulong rva,                    -- ADDRESS of 'template' for this type
		ushort size                   -- size of type, in bytes

   STRUCTURE   -- Data regions associated with a defined structure
	               Indexed by id
   	ulong id,                     -- unique number in sequence
		char name[],                  -- name of STRUCTURE
		ushort size                   -- size of STRUCTURE in bytes
	
	STRUCT_MEMBER -- Member types for a STRUCTURE
	                 Indexed by id, structure
   	ulong id,                     -- unique number in sequence
		ulong type,                   -- id of DATA_TYPE associated with this
		ulong structure,              -- id of STRUCTURE associated with this
		int order,                    -- which member # is this
		char name[]                   -- name of this struct member

	DATA_TYPE	-- Name/size for predefined data types
	               Indexed by id
   	ulong id,                     -- unique number in sequence
		int size,                     -- size of this DATA_TYPE
		char name[]                   -- name identifying this type
		
   CONSTANT    -- User-defined enumerations [e.g. #define]
	               Indexed by id, name, value
   	ulong id,                     -- unique number in sequence
		char name[],                  -- name identifying this constant value
		long value                    -- value of CONSTANT

   SEER_MACRO  -- User-defined SeerC scripts [strings or files] 
	               Indexed by id and name
   	ulong id,                     -- unique number in sequence
		int type,                     -- type of macro [string or file]
		char name[],                  -- name identifying this macro
		char macro[]                  -- text of macro/name of file

		Seer_Macro Types:
			SEERC_CODE   		0x01
			SEERC_FILE   		0x02


================================================================================
Responsible Parties

mammon_   -- design, coding, documentation, project mgt
ReZiDeNt  -- catalyst, initial co-design 
grugq     -- makefile
mo_k      -- x86 disassembler library co-design 
FatboyJoe -- front-end pioneering, code review, bug fixes
